/*
 *  TOPPERS Software
 *      Toyohashi Open Platform for Embedded Real-Time Systems
 *
 *  Copyright (C) 2005-2008 by TAKAGI Nobuhisa  
 * 
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 * 
 */

/*!
 *  \file   toppers/s_record.hpp
 *  \brief  ȥS쥳ɤ򰷤
 *
 *  Υե륯饹
 *  \code
 *  class s_record;
 *  struct s_record::record;
 *  class s_record::format_error;
 *  class s_record::checksum_error;
 *  \endcode
 */
#ifndef TOPPERS_S_RECORD_HPP_
#define TOPPERS_S_RECORD_HPP_

#include "toppers/config.hpp"
#include <iosfwd>
#include <string>
#include <vector>
#include <utility>
#include <stdexcept>
#include <boost/cstdint.hpp>

namespace toppers
{

  /*!
   *  \class  s_record s_record.hpp "toppers/s_recored.hpp"
   *  \brief  ȥS쥳ɤ򰷤Υ饹
   *
   *  \sa s_record::record, s_record::format_error, s_record::checksum_error
   */
  class s_record
  {
  public:
    typedef std::pair< unsigned long, std::vector< unsigned char > > value_type;
    typedef std::vector< value_type >::size_type size_type;

    /*!
     *  \struct record s_record.hpp "toppers/s_record.hpp"
     *  \brief  S쥳ɤΣԥ쥳ɾǼ뤿ι¤
     */
    struct record
    {
      int type;
      int length;
      unsigned long address;
      std::vector< unsigned char > data;
      int checksum;
    };

    /*!
     *  \class  data_error s_record.hpp "toppers/s_record.hpp"
     *  \brief  S쥳ɤΥǡ顼㳰饹
     */
    class data_error : public std::runtime_error
    {
    public:
      data_error() : std::runtime_error( "S-record data error" ) {}
    };
    /*!
     *  \class  format_error s_record.hpp "toppers/s_record.hpp"
     *  \brief  S쥳ɤν񼰥顼㳰饹
     */
    class format_error : public std::runtime_error
    {
    public:
      format_error() : std::runtime_error( "S-record format error" ) {}
    };
    /*!
     *  \class  checksum_error s_record.hpp "toppers/s_record.hpp"
     *  \brief  S쥳ɤΥå२顼㳰饹
     */
    class checksum_error : public std::runtime_error
    {
    public:
      checksum_error() : std::runtime_error( "S-record checksum error" ) {}
    };

    /*!
     *  \brief  ǥեȥ󥹥ȥ饯
     */
    s_record() : cache_( data_.end() ) {}
    /*!
     *  \brief  󥹥ȥ饯
     *  \param  istr  ϥȥ꡼
     *
     *  s_record 饹Ʊ˥ǡΥɤԤޤ
     */
    explicit s_record( std::istream& istr ) { load( istr ); }
    /*!
     *  \brief  ǥȥ饯
     */
    virtual ~s_record() {}

    void load( std::istream& istr );
    int operator[]( size_type address ) const;
    boost::uintmax_t get_value( std::size_t base, std::size_t size, bool little_endian ) const;
    unsigned long lower_bound() const;
    unsigned long upper_bound() const;
  protected:
    static record const read_record( std::string const& rec_buf );
    static int xdigit_to_int( int ch );
  private:
    std::vector< value_type > data_;
    mutable std::vector< value_type >::const_iterator cache_;
  };

}

#endif  // ! TOPPERS_S_RECORD_HPP_
